/*
    Socle - Socket Library Ecosystem
    Copyright (c) 2014, Ales Stibal <astib@mag0.net>, All rights reserved.

    This library  is free  software;  you can redistribute  it and/or
    modify  it  under   the  terms of the  GNU Lesser  General Public
    License  as published by  the   Free Software Foundation;  either
    version 3.0 of the License, or (at your option) any later version.
    This library is  distributed  in the hope that  it will be useful,
    but WITHOUT ANY WARRANTY;  without  even  the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
    
    See the GNU Lesser General Public License for more details.
    
    You  should have received a copy of the GNU Lesser General Public
    License along with this library.
*/

#include <socle_common.hpp>

#ifndef USE_OPENSSL300
#include <sslcom_dh.hpp>


DH *get_dh512()
        {
        static unsigned char dh512_p[]={
                0xE1,0x5A,0xAF,0x8D,0xDF,0x2E,0xAB,0x2B,0x6B,0xD3,0x9D,0x2B,
                0xB7,0x03,0x81,0x0A,0x04,0xD3,0x6A,0x8F,0xE6,0x09,0xC2,0x92,
                0x1F,0x70,0x79,0x78,0x1D,0xB0,0x54,0xC6,0x9C,0x66,0x16,0xFC,
                0x71,0xB2,0x64,0x84,0xB3,0xBD,0x7D,0x2A,0x24,0x9E,0x80,0x0E,
                0xA7,0xCB,0xF8,0xDF,0xA8,0xDF,0x00,0x69,0x98,0x5E,0x89,0x19,
                0xE2,0xA2,0xF7,0x1B,
                };
        static unsigned char dh512_g[]={
                0x02,
                };
        DH *dh;

        if ((dh=DH_new()) == nullptr) return nullptr;

#ifdef USE_OPENSSL11

        DH_set0_pqg(dh, BN_bin2bn( dh512_p, sizeof(dh512_p), nullptr),
                nullptr,
                BN_bin2bn( dh512_g, sizeof(dh512_g), nullptr));

        return dh;
#else
        dh->p=BN_bin2bn(dh512_p, sizeof(dh512_p), nullptr);
        dh->g=BN_bin2bn(dh512_g, sizeof(dh512_g), nullptr);
        if ((dh->p == nullptr) || (dh->g == nullptr))
                { DH_free(dh); return nullptr; }
        return(dh);
#endif
        }
/*
-----BEGIN DH PARAMETERS-----
MEYCQQDhWq+N3y6rK2vTnSu3A4EKBNNqj+YJwpIfcHl4HbBUxpxmFvxxsmSEs719
KiSegA6ny/jfqN8AaZheiRniovcbAgEC
-----END DH PARAMETERS-----
*/

DH *get_dh768()
        {
        static unsigned char dh768_p[]={
                0xA7,0x08,0xCF,0xD7,0x88,0x9F,0x70,0x67,0xD8,0x81,0xF4,0x6F,
                0x34,0xD2,0x25,0x29,0xA7,0xF2,0x55,0x12,0xE5,0x69,0x1A,0x4B,
                0x7C,0xD9,0x1A,0xC9,0x38,0xC0,0x77,0x58,0xEF,0xEF,0x83,0xE0,
                0x5B,0x21,0xAA,0x59,0x6D,0x3A,0x19,0x7B,0x07,0x99,0xC6,0x96,
                0x10,0x78,0x53,0x8B,0x05,0xFC,0x24,0xFA,0x3D,0xB2,0xE8,0xF6,
                0xA4,0xDC,0x7F,0xD1,0x18,0x7C,0x88,0x54,0xFD,0x58,0x27,0x6C,
                0x8E,0x95,0x5B,0x50,0xA2,0x9C,0x14,0xAF,0xBA,0xDE,0xCC,0xF1,
                0x91,0xFA,0xFF,0xE3,0x0E,0xDA,0x27,0x3F,0xE2,0x46,0xFB,0xE3,
                };
        static unsigned char dh768_g[]={
                0x02,
                };
        DH *dh;

        if ((dh=DH_new()) == nullptr) return nullptr;

#ifdef USE_OPENSSL11

        DH_set0_pqg(dh, BN_bin2bn( dh768_p, sizeof(dh768_p), nullptr),
                    nullptr,
                    BN_bin2bn( dh768_g, sizeof(dh768_g), nullptr));

        return dh;
#else
        dh->p=BN_bin2bn(dh768_p, sizeof(dh768_p), nullptr);
        dh->g=BN_bin2bn(dh768_g, sizeof(dh768_g), nullptr);
        if ((dh->p == NULL) || (dh->g == nullptr))
                { DH_free(dh); return nullptr; }
        return(dh);
#endif
        }
/*
-----BEGIN DH PARAMETERS-----
MGYCYQCnCM/XiJ9wZ9iB9G800iUpp/JVEuVpGkt82RrJOMB3WO/vg+BbIapZbToZ
eweZxpYQeFOLBfwk+j2y6Pak3H/RGHyIVP1YJ2yOlVtQopwUr7rezPGR+v/jDton
P+JG++MCAQI=
-----END DH PARAMETERS-----
*/

DH *get_dh1024()
        {
        static unsigned char dh1024_p[]={
                0x9F,0x79,0x24,0x87,0x4E,0x9E,0xDC,0x1E,0xDC,0x3B,0x45,0xAB,
                0x30,0x9C,0x40,0xCA,0xDC,0x4A,0x42,0xB0,0x57,0x84,0xBC,0xE1,
                0xB5,0x36,0x2D,0x46,0x5E,0xBE,0x8E,0xD0,0xB8,0x20,0x3F,0x6B,
                0x17,0x64,0xFD,0xD3,0xC0,0x60,0x08,0x2E,0xC9,0x4B,0x94,0x56,
                0x38,0x51,0x97,0x0C,0x27,0x89,0x44,0x13,0xA6,0xB1,0x41,0xA9,
                0xB3,0x1A,0x63,0x4C,0x13,0xC4,0xAD,0xB2,0xFC,0x57,0x6A,0x39,
                0x6D,0x37,0x0D,0x6D,0x45,0x0F,0x1B,0x40,0xFD,0xC3,0xC5,0x11,
                0xD6,0x73,0x88,0x49,0xB9,0x1D,0x46,0xCE,0x56,0x1A,0xEC,0xD2,
                0x4C,0x7F,0xB8,0x0F,0x20,0xEA,0x4E,0x66,0x8C,0xB2,0x93,0xCB,
                0xCD,0x17,0x3D,0x8E,0xA0,0xEB,0x98,0x6B,0xC8,0x43,0xDC,0x45,
                0xE9,0x75,0x5E,0x73,0xC0,0x8A,0x4A,0x7B,
                };
        static unsigned char dh1024_g[]={
                0x02,
                };
        DH *dh;

        if ((dh=DH_new()) == nullptr) return nullptr;

#ifdef USE_OPENSSL11

        DH_set0_pqg(dh, BN_bin2bn( dh1024_p, sizeof(dh1024_p), nullptr),
                    nullptr,
                    BN_bin2bn( dh1024_g, sizeof(dh1024_g), nullptr));

        return dh;
#else
        dh->p=BN_bin2bn(dh1024_p,sizeof(dh1024_p),nullptr);
        dh->g=BN_bin2bn(dh1024_g,sizeof(dh1024_g),nullptr);
        if ((dh->p == nullptr) || (dh->g == nullptr))
                { DH_free(dh); return nullptr; }
        return(dh);
#endif
        }
/*
-----BEGIN DH PARAMETERS-----
MIGHAoGBAJ95JIdOntwe3DtFqzCcQMrcSkKwV4S84bU2LUZevo7QuCA/axdk/dPA
YAguyUuUVjhRlwwniUQTprFBqbMaY0wTxK2y/FdqOW03DW1FDxtA/cPFEdZziEm5
HUbOVhrs0kx/uA8g6k5mjLKTy80XPY6g65hryEPcRel1XnPAikp7AgEC
-----END DH PARAMETERS-----
*/

DH *get_dh1536()
        {
        static unsigned char dh1536_p[]={
                0x9A,0xA6,0xDB,0x93,0x5C,0x29,0x3D,0x51,0x12,0x04,0xDD,0x41,
                0x72,0xDA,0x54,0xB4,0x21,0x76,0xF1,0xBD,0xB8,0x35,0xDA,0x5E,
                0xD8,0x38,0x73,0x34,0x8F,0xA6,0x2A,0x38,0x99,0x52,0xF8,0x4A,
                0xE4,0x15,0x19,0x4A,0x69,0x5E,0xA3,0x73,0x51,0xB1,0x90,0x97,
                0x62,0xCF,0xF7,0x76,0x07,0x89,0x2E,0xB6,0xD3,0x71,0x7B,0xE2,
                0x32,0x90,0xA9,0xF1,0x2A,0x37,0x6B,0xFB,0xF9,0x7A,0xD8,0x56,
                0xB3,0xE0,0x5C,0x7C,0xAD,0xE3,0x4B,0x9B,0x7C,0xEE,0x3C,0xF9,
                0xEE,0xC7,0xFF,0xD1,0x26,0x1F,0x8F,0xE0,0x9B,0x11,0x53,0x75,
                0x2E,0x4E,0xED,0x48,0x52,0xAD,0x73,0x8D,0xBD,0x35,0xFB,0xE0,
                0xD4,0x8E,0xAB,0xAE,0xFA,0xAE,0x33,0xB7,0xB9,0x56,0xAB,0x6B,
                0xBE,0x59,0xDA,0x7E,0x55,0x2A,0x0F,0xFC,0x5E,0xEC,0x54,0xE9,
                0x3C,0x3A,0x61,0x48,0x03,0x18,0xCF,0x47,0x55,0x2B,0x03,0xE4,
                0x1B,0xF3,0xD7,0x5D,0x5E,0x66,0x53,0xF8,0xD7,0xD0,0x18,0x27,
                0x4F,0x30,0xB6,0x3D,0xA1,0x69,0x6C,0xA5,0xBA,0x0A,0x2E,0xB6,
                0x21,0xFC,0x77,0x2C,0xD4,0xD0,0x6E,0x13,0x4C,0xEF,0x03,0x84,
                0x2E,0xC1,0x79,0x92,0xAC,0x3C,0x4C,0x96,0xB8,0xF8,0x85,0xEB,
                };
        static unsigned char dh1536_g[]={
                0x02,
                };
        DH *dh;

        if ((dh=DH_new()) == nullptr) return nullptr;

#ifdef USE_OPENSSL11

        DH_set0_pqg(dh, BN_bin2bn( dh1536_p, sizeof(dh1536_p), nullptr),
                    nullptr,
                    BN_bin2bn( dh1536_g, sizeof(dh1536_g), nullptr));

        return dh;
#else
        dh->p=BN_bin2bn(dh1536_p, sizeof(dh1536_p), nullptr);
        dh->g=BN_bin2bn(dh1536_g, sizeof(dh1536_g), nullptr);
        if ((dh->p == nullptr) || (dh->g == nullptr))
                { DH_free(dh); return nullptr; }
        return(dh);
#endif
        }
/*
-----BEGIN DH PARAMETERS-----
MIHHAoHBAJqm25NcKT1REgTdQXLaVLQhdvG9uDXaXtg4czSPpio4mVL4SuQVGUpp
XqNzUbGQl2LP93YHiS6203F74jKQqfEqN2v7+XrYVrPgXHyt40ubfO48+e7H/9Em
H4/gmxFTdS5O7UhSrXONvTX74NSOq676rjO3uVara75Z2n5VKg/8XuxU6Tw6YUgD
GM9HVSsD5Bvz111eZlP419AYJ08wtj2haWylugoutiH8dyzU0G4TTO8DhC7BeZKs
PEyWuPiF6wIBAg==
-----END DH PARAMETERS-----
*/


DH *get_dh2048()
        {
        static unsigned char dh2048_p[]={
                0x93,0x04,0x46,0x1C,0xB5,0x0B,0x33,0x36,0x8E,0x4B,0xE1,0x5E,
                0x2B,0x1A,0xD5,0xA5,0x1F,0x0B,0x79,0xFC,0xB0,0x6C,0x7D,0x75,
                0x85,0xBA,0xFB,0x67,0x5D,0x61,0xFD,0x86,0x33,0x28,0xFE,0xB1,
                0x97,0xE1,0xF2,0xA0,0x4A,0x51,0xFC,0x93,0xED,0x04,0x4F,0xFC,
                0x81,0x01,0xBC,0x7F,0x71,0x8A,0x9C,0x5A,0xD3,0x8C,0x88,0x12,
                0xA5,0x19,0xE1,0x29,0x79,0xBF,0x88,0x07,0xB0,0xE7,0x32,0x8D,
                0x0D,0xE0,0x3F,0xE0,0x26,0xA2,0xB2,0xC0,0xB1,0x0D,0xA4,0xFE,
                0x30,0xB0,0x35,0x1A,0x7B,0xE7,0x99,0xF3,0x26,0xB4,0xA5,0x9B,
                0x46,0xFC,0xB1,0xC8,0xFB,0x8A,0xF5,0x40,0x4C,0x31,0x06,0x60,
                0x94,0xA9,0xE3,0xD8,0x88,0xFE,0x64,0x6B,0xEF,0xA6,0xB2,0x8A,
                0x12,0x23,0x07,0x35,0xBD,0x87,0xC3,0x7A,0xC8,0x81,0xB6,0xC2,
                0x88,0x66,0xC2,0xE0,0x95,0xB1,0x28,0x5C,0x8E,0x41,0x92,0x9C,
                0x9B,0x92,0xEA,0x2D,0x76,0x31,0x1A,0x1E,0x17,0xF2,0xE3,0x2F,
                0x5B,0x14,0x80,0x1B,0x3F,0x81,0x52,0x1B,0x79,0xB0,0x40,0xF0,
                0x48,0x83,0x8E,0xA2,0xCD,0x42,0x37,0xFE,0xF5,0x54,0xB7,0xBB,
                0x6D,0xE3,0xA2,0x8E,0x66,0xC8,0x26,0x97,0x3A,0x52,0xA4,0x08,
                0xF3,0xF1,0x61,0x1F,0xDE,0xF9,0xB5,0xC5,0x19,0x7B,0xAA,0x12,
                0xA8,0xC3,0x0E,0xFD,0x26,0xD6,0x08,0x17,0xF2,0x11,0x4D,0xCF,
                0xD5,0xDA,0xB5,0x7C,0x7B,0x12,0xAB,0x64,0xB5,0x58,0xB4,0x47,
                0x00,0x95,0x3B,0xC8,0x51,0x39,0xB6,0xE7,0x6B,0x55,0xCE,0x55,
                0xC2,0xBE,0x8B,0x41,0x62,0x97,0x67,0x0C,0xEA,0x2A,0x30,0xB8,
                0xC5,0xDE,0xB8,0x4B,
                };
        static unsigned char dh2048_g[]={
                0x02,
                };
        DH *dh;

        if ((dh=DH_new()) == nullptr) return nullptr;

#ifdef USE_OPENSSL11

        DH_set0_pqg(dh, BN_bin2bn( dh2048_p, sizeof(dh2048_p), nullptr),
                        nullptr,
                        BN_bin2bn( dh2048_g, sizeof(dh2048_g), nullptr));

        return dh;
#else
        dh->p=BN_bin2bn(dh2048_p, sizeof(dh2048_p), nullptr);
        dh->g=BN_bin2bn(dh2048_g, sizeof(dh2048_g), nullptr);
        if ((dh->p == nullptr) || (dh->g == nullptr))
                { DH_free(dh); return nullptr; }
        return(dh);

#endif
        }

/*        
-----BEGIN DH PARAMETERS-----
MIIBCAKCAQEAkwRGHLULMzaOS+FeKxrVpR8LefywbH11hbr7Z11h/YYzKP6xl+Hy
oEpR/JPtBE/8gQG8f3GKnFrTjIgSpRnhKXm/iAew5zKNDeA/4CaissCxDaT+MLA1
GnvnmfMmtKWbRvyxyPuK9UBMMQZglKnj2Ij+ZGvvprKKEiMHNb2Hw3rIgbbCiGbC
4JWxKFyOQZKcm5LqLXYxGh4X8uMvWxSAGz+BUht5sEDwSIOOos1CN/71VLe7beOi
jmbIJpc6UqQI8/FhH975tcUZe6oSqMMO/SbWCBfyEU3P1dq1fHsSq2S1WLRHAJU7
yFE5tudrVc5Vwr6LQWKXZwzqKjC4xd64SwIBAg==
-----END DH PARAMETERS-----
*/

DH *get_dh3072()
        {
        static unsigned char dh3072_p[]={
                0xDA,0xD6,0xB8,0xE6,0x49,0x45,0xC1,0xFA,0xE7,0x13,0x2B,0xAC,
                0x63,0x63,0x32,0x2F,0x72,0x35,0xF9,0x11,0x20,0x2B,0x7F,0xCD,
                0x9A,0x13,0x1F,0x2B,0x2B,0x82,0x89,0xE4,0x16,0x4E,0xD2,0x63,
                0x2F,0x7A,0xA4,0xE0,0xBF,0x81,0xED,0x5C,0x5B,0xCA,0xD3,0xED,
                0x71,0xEC,0xA8,0xBF,0x36,0x58,0x8E,0x02,0x48,0xA3,0x81,0x7D,
                0x50,0xC4,0x0C,0xB4,0x4A,0x4D,0xC1,0x15,0x01,0x94,0x88,0x7A,
                0xF5,0x04,0x62,0x4D,0x35,0xFA,0x7D,0x45,0x45,0xC2,0x0C,0x4D,
                0x79,0xB3,0xEC,0x4B,0x2F,0xD1,0xFA,0xF7,0xBD,0xE3,0xFB,0xE3,
                0xA9,0x3C,0x52,0xA7,0x08,0x59,0x2A,0x06,0x65,0xAB,0x26,0x07,
                0x05,0xBB,0x39,0x6F,0x4D,0x68,0x9E,0x10,0x93,0x60,0x82,0x82,
                0xE7,0x8B,0x7A,0xF3,0x27,0x47,0xBA,0xF3,0x4F,0xA1,0xD9,0x5A,
                0x67,0x68,0xA0,0x02,0x99,0xBB,0xC1,0xC3,0xC2,0x57,0x3B,0x51,
                0x2E,0x53,0x55,0x05,0x61,0x0A,0x08,0x9D,0xD2,0xF6,0x3B,0xDA,
                0x5E,0x93,0xAD,0x44,0x85,0xD6,0x7F,0xEA,0x70,0x9E,0x08,0x26,
                0xCA,0x32,0xD9,0xAB,0x74,0xEB,0x70,0x4F,0x39,0xFC,0x69,0x2C,
                0x2F,0x0B,0x62,0xEA,0x37,0xC1,0x2C,0xF5,0xED,0x3C,0x54,0x5D,
                0x37,0x87,0x5C,0xA4,0xFB,0xC8,0x18,0x3C,0x0F,0xCB,0xC9,0xD0,
                0x67,0x96,0xD5,0x90,0xC0,0x9A,0xD0,0x80,0x05,0x58,0x6A,0x7C,
                0xA8,0xDD,0xDA,0x23,0x4A,0x37,0xE9,0x01,0x17,0x66,0xA0,0x30,
                0x17,0xA2,0x23,0xC8,0xE6,0x07,0x0D,0x13,0x38,0xCC,0x78,0xF8,
                0x19,0x23,0x86,0xA7,0x88,0xFA,0xD4,0x7F,0x62,0xE9,0xC6,0xE8,
                0xB0,0x6C,0x63,0xBD,0x0E,0xB6,0x81,0x24,0xA7,0x03,0xCC,0x67,
                0xC1,0x81,0xE3,0xA1,0x01,0xE8,0x89,0x0E,0x2A,0xD1,0x04,0xB7,
                0x3C,0x05,0x64,0xC6,0x48,0xBF,0x59,0x84,0xCE,0x3C,0x0A,0x19,
                0x46,0x32,0x68,0x18,0x25,0x0B,0x55,0x43,0xDD,0x6B,0x4D,0x7C,
                0x30,0x9B,0x7A,0xAF,0x7A,0x4C,0x69,0xDE,0x8E,0xE9,0xCD,0x52,
                0x2F,0x02,0xA2,0x08,0x15,0x45,0x94,0xAB,0x07,0xA0,0x20,0xF7,
                0xE4,0xBA,0xF0,0xBD,0x73,0x56,0x3D,0x50,0x39,0x3C,0x07,0x27,
                0x1C,0xEF,0x69,0x53,0x41,0xD1,0x83,0x7F,0xF8,0x2D,0xC8,0xEE,
                0xF5,0xE3,0x4B,0x87,0x9B,0x8C,0x78,0xEC,0x59,0x28,0xFF,0x45,
                0x67,0x0C,0x4F,0xB7,0x11,0x68,0xB1,0xE2,0x37,0x95,0xD3,0xB1,
                0xD0,0x1B,0x96,0x0F,0xC3,0xE2,0x51,0x82,0x5B,0x42,0xE2,0x0B,
                };
        static unsigned char dh3072_g[]={
                0x02,
                };
        DH *dh;

        if ((dh=DH_new()) == nullptr) return nullptr;

#ifdef USE_OPENSSL11

    DH_set0_pqg(dh, BN_bin2bn( dh3072_p, sizeof(dh3072_p), nullptr),
                nullptr,
                BN_bin2bn( dh3072_g, sizeof(dh3072_g), nullptr));

    return dh;
#else

    dh->p=BN_bin2bn(dh3072_p, sizeof(dh3072_p), nullptr);
        dh->g=BN_bin2bn(dh3072_g, sizeof(dh3072_g), nullptr);
        if ((dh->p == nullptr) || (dh->g == nullptr))
                { DH_free(dh); return nullptr; }
        return(dh);

#endif
        }

/*
-----BEGIN DH PARAMETERS-----
MIIBiAKCAYEA2ta45klFwfrnEyusY2MyL3I1+REgK3/NmhMfKyuCieQWTtJjL3qk
4L+B7VxbytPtceyovzZYjgJIo4F9UMQMtEpNwRUBlIh69QRiTTX6fUVFwgxNebPs
Sy/R+ve94/vjqTxSpwhZKgZlqyYHBbs5b01onhCTYIKC54t68ydHuvNPodlaZ2ig
Apm7wcPCVztRLlNVBWEKCJ3S9jvaXpOtRIXWf+pwnggmyjLZq3TrcE85/GksLwti
6jfBLPXtPFRdN4dcpPvIGDwPy8nQZ5bVkMCa0IAFWGp8qN3aI0o36QEXZqAwF6Ij
yOYHDRM4zHj4GSOGp4j61H9i6cbosGxjvQ62gSSnA8xnwYHjoQHoiQ4q0QS3PAVk
xki/WYTOPAoZRjJoGCULVUPda018MJt6r3pMad6O6c1SLwKiCBVFlKsHoCD35Lrw
vXNWPVA5PAcnHO9pU0HRg3/4Lcju9eNLh5uMeOxZKP9FZwxPtxFoseI3ldOx0BuW
D8PiUYJbQuILAgEC
-----END DH PARAMETERS-----
*/

DH *get_dh4096()
        {
        static unsigned char dh4096_p[]={
                0xB5,0x13,0xEE,0x0B,0x33,0x08,0xE6,0x6B,0xA0,0x73,0x86,0x59,
                0xAB,0xDD,0xE4,0x08,0xAE,0x45,0xD3,0xA4,0x72,0x68,0x62,0xEC,
                0xCA,0x71,0x35,0xF4,0xA6,0x46,0xE2,0x0E,0xE5,0x03,0x15,0xFE,
                0xAA,0x50,0xE3,0x6E,0x35,0x69,0x55,0xBD,0xB7,0x9E,0xE6,0xD9,
                0x2C,0xC3,0x90,0x84,0xAF,0x9C,0xAD,0x7F,0xDA,0x6A,0x70,0x82,
                0x93,0x07,0xC9,0x3F,0x2A,0x69,0x2B,0x15,0xAE,0x0C,0x84,0xEA,
                0x9C,0x5D,0xA2,0x6C,0x82,0xBF,0xA1,0x9E,0x86,0x20,0x8C,0xE7,
                0x06,0x7B,0x37,0x00,0x37,0x6D,0xB2,0x4C,0x54,0x5F,0x5D,0xCC,
                0xB5,0xE0,0xE4,0x37,0xB2,0x95,0xD7,0x3A,0x09,0x7F,0x18,0x9A,
                0xC0,0x66,0x3B,0x71,0x52,0x62,0x25,0x07,0xA1,0xC8,0xC8,0x65,
                0x2D,0x1C,0x9C,0x24,0x6D,0x27,0xDA,0xA1,0x1D,0xA2,0x41,0x00,
                0x56,0x91,0xD0,0xE4,0x32,0x2F,0x2C,0xC4,0xE7,0x9B,0xD6,0x48,
                0xA1,0xF8,0xC3,0x10,0x7F,0xC5,0x33,0x63,0x10,0x07,0xC8,0x92,
                0x1D,0x5D,0xAE,0xDB,0x0A,0x1F,0x2A,0xE9,0xF4,0xDB,0x17,0xD0,
                0xF2,0x4C,0xE1,0xCA,0x90,0xB1,0x47,0xE7,0x6F,0x09,0x26,0x03,
                0x3B,0x7E,0x08,0x9E,0xC5,0x48,0x18,0xD5,0x00,0xE6,0xD7,0x18,
                0xE8,0x2F,0x36,0x2E,0x48,0x08,0xC5,0x3C,0x28,0xC5,0x43,0xF6,
                0x12,0x5B,0x86,0xDC,0x87,0x4B,0xCA,0xD8,0x7D,0x87,0x6E,0x94,
                0xB8,0x4C,0xA8,0x8F,0x86,0x44,0x3D,0xD8,0x40,0x4F,0x31,0xEF,
                0xE5,0x10,0xB7,0x5E,0xAF,0xC8,0xF2,0x56,0x75,0xFD,0x4F,0xAF,
                0x80,0x0C,0x3A,0x0F,0xA2,0xD4,0x8E,0x59,0x45,0xBC,0xF0,0x86,
                0x7F,0xBF,0xB4,0xE8,0xEE,0x8C,0xAB,0x93,0x12,0x84,0xBE,0xC3,
                0x6B,0x68,0x04,0xFA,0xEB,0x59,0x10,0xE3,0x2F,0x71,0xCF,0x34,
                0x3F,0xDD,0x3A,0x96,0x45,0x9A,0x84,0x11,0x81,0x85,0x77,0x20,
                0xAF,0x52,0x1D,0xDB,0xD3,0xF3,0x57,0xD5,0xC3,0xFB,0x0F,0x22,
                0x32,0xE1,0xC9,0x8D,0x9B,0x6C,0x97,0x72,0x98,0xE1,0xF3,0x44,
                0x55,0x2B,0xAE,0xF8,0x5E,0xB4,0x1B,0xE2,0x7D,0x36,0x5B,0x7F,
                0xAA,0x80,0x15,0xE5,0x67,0x17,0x03,0xF5,0x00,0x9D,0x75,0xF8,
                0x30,0x1B,0x08,0x2A,0xEF,0x4B,0x73,0xEA,0xD4,0xAF,0x64,0x11,
                0x7B,0xEF,0x1C,0x2C,0xCA,0x1F,0xBF,0x61,0x49,0x4C,0x53,0x01,
                0x1F,0x35,0xFC,0xD3,0xC5,0xE9,0xAF,0x38,0x6C,0xD8,0x52,0xB0,
                0xFC,0xE6,0xB4,0x10,0xB7,0xC3,0x01,0x55,0xEB,0xF0,0x88,0x1F,
                0x39,0x19,0x2D,0x73,0xE2,0x9E,0x19,0x3B,0x05,0x0B,0x5E,0xE8,
                0xBB,0xEB,0x7E,0x3A,0x23,0x19,0x5F,0x71,0xD7,0xA9,0xE9,0xC0,
                0xBC,0x79,0x10,0x37,0xBA,0x4C,0x72,0xA9,0x20,0xA9,0x9A,0x84,
                0x21,0x51,0x58,0xE2,0x01,0x51,0xED,0xB6,0x42,0xA5,0x81,0x11,
                0x0F,0x28,0xFE,0x2B,0x6D,0x9A,0xF2,0x4F,0x5E,0x11,0x78,0xA7,
                0xED,0x89,0x7F,0xD6,0xE2,0x6C,0x47,0x66,0x94,0x09,0x14,0x17,
                0x4E,0xF1,0x32,0x74,0xEE,0xB8,0x38,0x1B,0x14,0x80,0x51,0xF4,
                0x08,0x4C,0x8F,0x02,0x82,0x49,0x80,0xA5,0x69,0xC8,0xF0,0x34,
                0x52,0x3C,0x65,0x18,0xD1,0xA3,0x67,0x7C,0x48,0x83,0x6B,0x18,
                0x69,0x68,0xDB,0x87,0x25,0x83,0xAA,0xA8,0xA0,0xCA,0xB9,0x9A,
                0x25,0x8D,0x21,0xBB,0x72,0x9D,0xCA,0xB3,
                };
        static unsigned char dh4096_g[]={
                0x02,
                };
        DH *dh;

        if ((dh=DH_new()) == NULL) return(NULL);

#ifdef USE_OPENSSL11

    DH_set0_pqg(dh, BN_bin2bn( dh4096_p, sizeof(dh4096_p), nullptr),
                nullptr,
                BN_bin2bn( dh4096_g, sizeof(dh4096_g), nullptr));

    return dh;
#else

    dh->p=BN_bin2bn(dh4096_p,sizeof(dh4096_p),NULL);
        dh->g=BN_bin2bn(dh4096_g,sizeof(dh4096_g),NULL);
        if ((dh->p == NULL) || (dh->g == NULL))
                { DH_free(dh); return(NULL); }
        return(dh);

#endif
        }
/*
-----BEGIN DH PARAMETERS-----
MIICCAKCAgEAtRPuCzMI5mugc4ZZq93kCK5F06RyaGLsynE19KZG4g7lAxX+qlDj
bjVpVb23nubZLMOQhK+crX/aanCCkwfJPyppKxWuDITqnF2ibIK/oZ6GIIznBns3
ADdtskxUX13MteDkN7KV1zoJfxiawGY7cVJiJQehyMhlLRycJG0n2qEdokEAVpHQ
5DIvLMTnm9ZIofjDEH/FM2MQB8iSHV2u2wofKun02xfQ8kzhypCxR+dvCSYDO34I
nsVIGNUA5tcY6C82LkgIxTwoxUP2EluG3IdLyth9h26UuEyoj4ZEPdhATzHv5RC3
Xq/I8lZ1/U+vgAw6D6LUjllFvPCGf7+06O6Mq5MShL7Da2gE+utZEOMvcc80P906
lkWahBGBhXcgr1Id29PzV9XD+w8iMuHJjZtsl3KY4fNEVSuu+F60G+J9Nlt/qoAV
5WcXA/UAnXX4MBsIKu9Lc+rUr2QRe+8cLMofv2FJTFMBHzX808Xprzhs2FKw/Oa0
ELfDAVXr8IgfORktc+KeGTsFC17ou+t+OiMZX3HXqenAvHkQN7pMcqkgqZqEIVFY
4gFR7bZCpYERDyj+K22a8k9eEXin7Yl/1uJsR2aUCRQXTvEydO64OBsUgFH0CEyP
AoJJgKVpyPA0UjxlGNGjZ3xIg2sYaWjbhyWDqqigyrmaJY0hu3KdyrMCAQI=
-----END DH PARAMETERS-----
*/

#endif